'use strict';

const Boom = require('boom');
const Joi = require('joi');
const semver = require('semver');
const match = require('semver-match');
const debug = require('debug')('hapi-semver-api:plugin');

const Package = require('./package');

const { routesAddVersions } = require('./routes.add.versions');

const internals = {
  optionsSchema: Joi.object({
    minValidVersion: Joi.string().required(),
    maxValidVersion: Joi.string().required(),
    defaultVersion: Joi.string().required(),
    versionHeader: Joi.string().trim().min(1).default('accept-version'),
    responseHeader: Joi.string().trim().min(1).default('content-version'),
    basePath: Joi.string().trim().min(1).default('/')
  })
};

function _extractVersionFromCustomHeader(request, options) {
  return request.headers[options.versionHeader] ? request.headers[options.versionHeader] : null;
}

function register(server, dirtyOptions, next) {
  debug('register plugin with options', dirtyOptions);

  const validateOptions = Joi.validate(dirtyOptions, internals.optionsSchema, { abortEarly: false, allowUnknown: false });

  if (validateOptions.error) {
    return next(validateOptions.error);
  }

  // Use the validated and maybe converted values from Joi
  const options = validateOptions.value;

  debug('options validated successfully');

  server.ext('onRequest', async (request, reply) => {
    const { path, method } = request;
    debug('Start check against "%s %s"', method, path);

    if (method === 'options') {
      debug('Method = options, skip version check');
      return reply.continue();
    }

    let expectedVersion = _extractVersionFromCustomHeader(request, options);
    if (expectedVersion) {
      if (!semver.validRange(expectedVersion)) {
        return reply(Boom.badRequest(`Header '${options.versionHeader}' has invalid semver version '${expectedVersion}'`));
      }
      debug('Extract expected version %s', expectedVersion);
    } else {
      expectedVersion = options.defaultVersion;
      debug('Use default version %s', expectedVersion);
    }

    const route = server.match(request.method, path);
    if (!route) { // no routes found, skip version check
      debug('No routes found, skip version check');
      return reply.continue();
    }

    if (!(route.settings && route.settings.plugins && route.settings.plugins.version && route.settings.plugins.version.list)) {
      debug('Route "%s %s" not configured with Version Plugin, skip');
      return reply.continue();
    }

    const availableVersions = route.settings.plugins.version.list;
    debug('Available versions', availableVersions);

    const matchedVersion = match(expectedVersion, availableVersions);
    if (!matchedVersion) {
      return reply(Boom.badRequest(`Version '${expectedVersion}' not found for route '${path}'. Available versions list: '${availableVersions.join(', ')}'`));
    }

    debug('Matched version %s', matchedVersion);

    request.pre.apiVersion = matchedVersion;

    const fullPath = request.url.path;

    request.setUrl(`/v${matchedVersion}${fullPath}`);

    return reply.continue();
  });

  server.ext('onPreResponse', (request, reply) => {
    const version = request.pre.apiVersion;

    const { response } = request;

    if (request.response.isBoom) {
      response.output.headers[options.responseHeader] = version;
    } else {
      response.header(options.responseHeader, version);
    }

    return reply.continue();
  });

  return next();
}

register.attributes = {
  name: Package.name,
  version: Package.version,
  multiple: false
};

module.exports = { register, routesAddVersions };
